@@ -18,47 +18,6 @@ import os
 CURR_RESULTS_VERSION = "2.0"
 
 
-def _get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles):
-    ordered_kps = []
-    cols = ['key point']
-    for title in titles:
-        n_comments_per_kp_per_title = results_to_kp_to_n_comments[title]
-        new_kps = set(n_comments_per_kp_per_title.keys()).difference(set(ordered_kps))
-        new_kps = sorted(new_kps, key= lambda x:n_comments_per_kp_per_title[x] ,reverse=True)
-        ordered_kps += new_kps
-        cols.extend([f"{title}_n_comments", f"{title}_precent"])
-
-    if "none" in ordered_kps:
-        ordered_kps.remove("none")
-
-    add_change = False
-    if len(titles) == 2:
-        cols.append("change_percent")
-        add_change = True
-
-    rows = []
-    title_to_precent = {}
-    total_row = ["total"]
-    for i,kp in enumerate(ordered_kps):
-        row = [kp]
-        for title in titles:
-            n_comments_title_kp = results_to_kp_to_n_comments[title].get(kp,0)
-            percent_comments_title_kp = (100*n_comments_title_kp/results_to_total_comments[title]) if n_comments_title_kp else 0
-            row.extend([n_comments_title_kp,f'{percent_comments_title_kp:.2f}%'])
-            title_to_precent[title] = percent_comments_title_kp
-            if i == 0:
-                total_row.extend([results_to_total_comments[title] , "1"])
-        if add_change:
-            change_percent = title_to_precent[titles[1]] - title_to_precent[titles[0]]
-            row.append(f'{change_percent:.2f}%')
-            if i==0:
-                total_row.append("")
-        rows.append(row)
-    rows.append(total_row)
-    comparison_df = pd.DataFrame(rows, columns=cols)
-    return comparison_df
-
-
 class KpsResult:
     """
     Class to hold and process the results of a KPS job.
@@ -68,10 +27,13 @@ class KpsResult:
         :param result_json: the result json returned from the key point client using get_results.
         """
         self.result_json = result_json
+        kps = self.get_key_points()
+        if len(kps) == 0:
+            logging.info("No key points found. Returning empty KpsResult.")
         self.result_df = self._create_result_df()
         self.version = CURR_RESULTS_VERSION
         self.stances = set(result_json["job_metadata"]["per_stance"].keys()).difference({"no-stance"})
-        self.filter_min_relations_for_text = filter_min_relations
+        self.filter_min_relations = filter_min_relations
         self.kp_id_to_hierarchical_data = self._get_kp_id_to_hierarchical_data()
         self.summary_df = self._result_df_to_summary_df()
 
@@ -95,7 +57,7 @@ class KpsResult:
             json.dump(self.result_json, f)
 
     @staticmethod
-    def load(json_file: str):
+    def load(json_file: str, filter_min_relations:Optional[float] = 0.4):
         """
         Load results from a json file
         :param json_file: the file to load the results from, obtained using the save() method.
@@ -104,7 +66,7 @@ class KpsResult:
         logging.info(f"Loading results from: {json_file}")
         with open(json_file, 'r') as f:
             json_res = json.load(f)
-            return KpsResult.create_from_result_json(json_res)
+            return KpsResult.create_from_result_json(json_res, filter_min_relations)
 
     def _create_result_df(self):
         sentences_data = self.result_json["sentences_data"]
@@ -221,20 +183,27 @@ class KpsResult:
         return pd.DataFrame(summary_rows, columns=summary_cols)
 
     @staticmethod
-    def create_from_result_json(result_json, filter_min_relations_for_text=0.4):
+    def create_from_result_json(result_json, filter_min_relations=0.4):
         """
         Create KpsResults from results_json
         :param result_json: the json object obtained from the client via "get_result" or "get_result_from_futures"
+        :param filter_min_relations: minimal relation score between key points to be considered related in the hierarchy
         :return: KpsResult object
         """
         if 'keypoint_matchings' not in result_json:
-            raise KpsIllegalInputException("Faulty results json provided: does not contain 'keypoint_matchings'. returning empty results")
+            raise KpsIllegalInputException("Faulty results json provided: does not contain 'keypoint_matching'.")
+        if "job_metadata" not in result_json:
+            raise KpsIllegalInputException("Faulty result_json provided: does not contain 'job_metadata'. result_json must be"
+                                           "the json object returned by the server when running with SDK version >= 5.0.0."
+                                           "Converting older json results is not supported. to generate the old"
+                                           "results for the given json object please use SDK version <= 4.3.2. To generate new result"
+                                           "please rerun the KPS job using the current SDK")
         else:
             try:
                 version = result_json.get("version", "1.0")
                 if version != CURR_RESULTS_VERSION:
                     result_json = KpsResult._convert_to_new_version(result_json, version, CURR_RESULTS_VERSION)
-                return KpsResult(result_json, filter_min_relations_for_text)
+                return KpsResult(result_json, filter_min_relations)
             except Exception as e:
                 logging.error("Could not create KpsResults from json.")
                 raise e
@@ -255,7 +224,6 @@ class KpsResult:
     def generate_docx_report(self, output_dir: str, result_name: str,
                              n_matches_in_docx: Optional[int] = 50,
                              include_match_score_in_docx: Optional[bool] = False,
-                             min_n_matches_in_docx: Optional[int] = 5,
                              kp_id_to_hierarchical_data=None):
         """
         creates <output_dir>/<result_name>_hierarchical.docx: This Microsoft Word document shows the key point hierarchy and matching sentences
@@ -264,7 +232,6 @@ class KpsResult:
         :param result_name: name of the results to appear in the output files.
         :param n_matches_in_docx: number of top matches to write in the textual summary (docx file). Pass None for all matches.
         :param include_match_score_in_docx: when set to true, the match score between the sentence and the key point is added.
-        :param min_n_matches_in_docx: remove key points with less than min_n_matches_in_docx matching sentences.
         :param kp_id_to_hierarchical_data: optional, should be set to None.
         """
         if not kp_id_to_hierarchical_data:
@@ -274,12 +241,11 @@ class KpsResult:
         save_hierarchical_graph_data_to_docx(full_result_df=self.result_df, kp_id_to_data=kp_id_to_hierarchical_data,
                                              result_filename=docx_file, meta_data=meta_data, n_matches=n_matches_in_docx,
                                              include_match_score=include_match_score_in_docx,
-                                             min_n_matches=min_n_matches_in_docx)
+                                             min_n_matches=0)
 
     def generate_docx_report_using_given_tree(self, full_results,
                                               output_dir, result_name, n_top_matches_in_graph=20,
-                                              n_matches_in_docx=50, include_match_score_in_docx=False,
-                                              min_n_matches_in_docx=5):
+                                              n_matches_in_docx=50, include_match_score_in_docx=False):
         '''
         Create hierarchical result for this result, using a precalculated hierarchical results.
         This is useful when we first create results using the whole data, and then want to calculate the
@@ -294,26 +260,23 @@ class KpsResult:
         :param n_top_matches_in_graph : optional, number of top matches to add to the graph_data file.
         :param n_matches_in_docx: optional, number of top matches to write in the textual summary (docx file). Pass None for all matches.
         :param include_match_score_in_docx: optional, when set to true, the match score between the sentence and the key point is added.
-        :param min_n_matches_in_docx: optional, remove key points with less than min_n_matches_in_docx matching sentences.
         '''
         graph_full_data = create_graph_data(full_results.result_df, full_results._get_number_of_unique_sentences())
         hierarchical_graph_full_data = graph_data_to_hierarchical_graph_data(graph_data=graph_full_data)
 
         new_hierarchical_graph_data = get_hierarchical_graph_from_tree_and_subset_results(
             hierarchical_graph_full_data,
-            self.result_df, self.filter_min_relations_for_text, n_top_matches_in_graph)
+            self.result_df, self.filter_min_relations, n_top_matches_in_graph)
 
-        new_kp_id_to_hierarchical_data = get_hierarchical_kps_data(self.result_df, new_hierarchical_graph_data, self.filter_min_relations_for_text)
+        new_kp_id_to_hierarchical_data = get_hierarchical_kps_data(self.result_df, new_hierarchical_graph_data, self.filter_min_relations)
         self.generate_docx_report(output_dir, result_name,
                             n_matches_in_docx=n_matches_in_docx,
                             include_match_score_in_docx=include_match_score_in_docx,
-                            min_n_matches_in_docx=min_n_matches_in_docx,
                             kp_id_to_hierarchical_data = new_kp_id_to_hierarchical_data)
 
     def export_to_all_outputs(self, output_dir: str, result_name: str,
                               n_matches_in_docx: Optional[int] = 50,
                               include_match_score_in_docx: Optional[bool] = False,
-                              min_n_matches_in_docx: Optional[int] = 5,
                               ):
         """
         Generates all the kps available output types.
@@ -321,7 +284,6 @@ class KpsResult:
         :param result_name: name of the results to appear in the output files.
         :param n_matches_in_docx: optional, number of top matches to write in the textual summary (docx file). Pass None for all matches.
         :param include_match_score_in_docx: optional, when set to true, the match score between the sentence and the key point is added.
-        :param min_n_matches_in_docx: optional, remove key points with less than min_n_matches_in_docx matching sentences.
         Creates 3 outout files:
              * <ouput_dir>/<result_name>.csv : full results as csv .
              * <ouput_dir>/<result_name>_kps_summary.csv : summary results as csv.
@@ -337,8 +299,7 @@ class KpsResult:
 
         self.generate_docx_report(output_dir, result_name,
                                   n_matches_in_docx=n_matches_in_docx,
-                                  include_match_score_in_docx=include_match_score_in_docx,
-                                  min_n_matches_in_docx=min_n_matches_in_docx)
+                                  include_match_score_in_docx=include_match_score_in_docx)
 
     def print_result(self, n_sentences_per_kp: int, title: str, n_top_kps:Optional[int]=None):
         '''
@@ -439,6 +400,53 @@ class KpsResult:
                     total_sentences = total_sentences.union(matching_sents_ids)
             return len(total_sentences)
 
+    def _get_comparison_df(self, results_to_kp_to_n_comments, results_to_total_comments, titles, kp_to_stance):
+        ordered_kps = []
+        cols = ['key point', 'stance']
+
+        for title in titles:
+            n_comments_per_kp_per_title = results_to_kp_to_n_comments[title]
+            new_kps = set(n_comments_per_kp_per_title.keys()).difference(set(ordered_kps))
+            new_kps = sorted(new_kps, key=lambda x: n_comments_per_kp_per_title[x], reverse=True)
+            ordered_kps += new_kps
+            cols.extend([f"{title}_n_comments", f"{title}_percent"])
+
+        if "none" in ordered_kps:
+            ordered_kps.remove("none")
+
+        add_change = False
+        if len(titles) == 2:
+            cols.append("change_percent")
+            add_change = True
+
+        rows = []
+        title_to_precent = {}
+        total_row = ["total", ""]
+        for i, kp in enumerate(ordered_kps):
+            row = [kp, kp_to_stance[kp]]
+
+            for title in titles:
+                n_comments_title_kp = results_to_kp_to_n_comments[title].get(kp, 0)
+                percent_comments_title_kp = (
+                            100 * n_comments_title_kp / results_to_total_comments[title]) if n_comments_title_kp else 0
+                row.extend([n_comments_title_kp, f'{percent_comments_title_kp:.2f}%'])
+                title_to_precent[title] = percent_comments_title_kp
+                if i == 0:
+                    total_row.extend([results_to_total_comments[title], "1"])
+            if add_change:
+                change_percent = title_to_precent[titles[1]] - title_to_precent[titles[0]]
+                row.append(f'{change_percent:.2f}%')
+                if i == 0:
+                    total_row.append("")
+
+            rows.append(row)
+        rows.append(total_row)
+        comparison_df = pd.DataFrame(rows, columns=cols)
+
+        if len(set(kp_to_stance.values()).difference({"no-stance", None})) == 0:
+            comparison_df = comparison_df[[c for c in comparison_df.columns if c != "stance"]]
+        return comparison_df
+
     def compare_with_comment_subsets(self, comments_subsets_dict:Dict[str, List[str]], include_full:Optional[bool] = True):
         """
         Compare the full result with the results generated from comment subsets. This is useful in order to compare the
@@ -457,7 +465,9 @@ class KpsResult:
         results_to_kp_to_n_comments = {"full": self._get_kp_to_n_matched_comments()} if include_full else {}
         results_to_kp_to_n_comments.update(
             {title: self._get_kp_to_n_matched_comments(comments_subset=comment_ids) for title, comment_ids in comments_subsets_dict.items()})
-        return _get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles)
+
+        kp_to_stance = self.get_kp_to_stance()
+        return self._get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles, kp_to_stance)
 
     def compare_with_other_results(self, this_title : str, other_results_dict : Dict[str,'KpsResult']):
         """
@@ -473,7 +483,12 @@ class KpsResult:
 
         results_to_kp_to_n_comments = {this_title:self._get_kp_to_n_matched_comments()}
         results_to_kp_to_n_comments.update({title:result._get_kp_to_n_matched_comments()  for title,result in other_results_dict.items()})
-        return _get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles)
+
+        kp_to_stance = self.get_kp_to_stance()
+        for result in other_results_dict.values():
+            kp_to_stance.update(result.get_kp_to_stance())
+
+        return self._get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles, kp_to_stance)
 
     @staticmethod
     def get_merged_pro_con_results(pro_result: 'KpsResult', con_result: 'KpsResult'):
@@ -483,6 +498,7 @@ class KpsResult:
         :param pro_result: KpsResult generated from running on stance "PRO".
         :param con_result: KpsResult generated from running on stance "CON".
         """
+        logging.info("Merging positive and negative KpsResults.")
         assert pro_result.stances == {"pro"}, f"pro_result must be generated from running on stance 'PRO', given {pro_result.stances}"
         assert con_result.stances == {"con"}, f"con_result must be generated from running on stance 'CON', given {con_result.stances}"
         con_meta = con_result.result_json["job_metadata"]["general"]
@@ -518,7 +534,8 @@ class KpsResult:
         combined_results_json = {'keypoint_matchings':keypoint_matchings, "sentences_data":sentences_data,
                                  "version":CURR_RESULTS_VERSION,
                                  "job_metadata":new_metadata}
-        return KpsResult.create_from_result_json(combined_results_json)
+        filter_min_relations = np.min([pro_result.filter_min_relations, con_result.filter_min_relations])
+        return KpsResult.create_from_result_json(combined_results_json, filter_min_relations=filter_min_relations)
 
     @staticmethod
     # If result_json is of the old server version, convert to version 2.0
@@ -591,10 +608,14 @@ class KpsResult:
             kp_n_comments[kp] = n_matches
         return kp_n_comments
 
+    def get_key_points(self):
+        return [keypoint_matching["keypoint"] for keypoint_matching in self.result_json['keypoint_matchings'] if keypoint_matching["keypoint"] != "none"]
+
+
     def _get_kp_id_to_hierarchical_data(self):
         graph_data = create_graph_data(self.result_df, n_sentences=self._get_number_of_unique_sentences())
-        hierarchical_graph_data = graph_data_to_hierarchical_graph_data(graph_data=graph_data)
-        return get_hierarchical_kps_data(self.result_df, hierarchical_graph_data, self.filter_min_relations_for_text)
+        hierarchical_graph_data = graph_data_to_hierarchical_graph_data(graph_data=graph_data, filter_min_relations=self.filter_min_relations)
+        return get_hierarchical_kps_data(self.result_df, hierarchical_graph_data, self.filter_min_relations)
 
     @staticmethod
     def _get_stance_from_server_stances(server_stances):
@@ -643,3 +664,7 @@ class KpsResult:
         elif len(self.stances) == 1:
             return list(self.stances)[0]
         return "pro and con"
+
+    def get_kp_to_stance(self):
+         keypoint_matchings = self.result_json["keypoint_matchings"]
+         return {keypoint_matching['keypoint']:keypoint_matching.get("stance", None) for keypoint_matching in keypoint_matchings}
